package MT::Plugin::SKR::Bookmark;
use strict;
use MT 4;
use MT::PluginData;

use vars qw( $MYNAME $VERSION );
$MYNAME = 'Bookmark';
$VERSION = '0.01 Preview';

use base qw( MT::Plugin );
my $plugin = __PACKAGE__->new({
        id => $MYNAME,
        key => $MYNAME,
        name => $MYNAME,
        version => $VERSION,
        author_name => 'SKYARC System Co.,Ltd.',
        author_link => 'http://www.skyarc.co.jp/',
        doc_link => 'http://www.skyarc.co.jp/engineerblog/entry/movabletypebookmark.html ',
        description => <<HTMLHEREDOC,
<__trans phrase="Add bookmark menu to access each actions speedy.">
HTMLHEREDOC
        l10n_class => $MYNAME. '::L10N',
});
MT->add_plugin( $plugin );

sub instance { $plugin; }

### Registry
sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        applications => {
            cms => {
                menus => {
                    'bookmark' => {
                        label => 'Bookmark',
                        order => 1000,
                    },
                    'bookmark:add' => {
                        label => '* Add/Edit',
                        mode => 'bookmark_this_page',
                        order => 0,
                    },
                },
                methods => {
                    bookmark_this_page => \&_hdlr_bookmark_this_page,
                    save_bookmark => \&_hdlr_save_bookmark,
                },
            },
        },
        callbacks => {
            'pre_run' => \&_hdlr_pre_run,
            'MT::Author::pre_remove' => \&_hdlr_pre_remove_author,
        },
    });
}



### Method - bookmark_this_page
sub _hdlr_bookmark_this_page {
    my ($app) = @_;

    my $user = $app->user;
    my $from_url = $app->param('url') || $ENV{HTTP_REFERER}
        or return 'Can&apos;t work. Please send me your referer.';

    my $data = load_plugindata (key_name ($user->id)) || {};

    my %param = (
        from_url => $from_url,
        title => $data->{$from_url} || '',
        already_saved => defined $data->{$from_url},
    );
    &instance->load_tmpl ('edit.tmpl', \%param);
}

### Method - save_bookmark
sub _hdlr_save_bookmark {
    my ($app) = @_;

    if (defined (my $title = $app->param('title')) && defined (my $url = $app->param('url'))) {
        my $user = $app->user;
        my $data = load_plugindata (key_name ($user->id)) || {};
        if ($app->param('action') eq 'save') {
            $data->{$url} = $title;
        }
        elsif ($app->param('action') eq 'delete') {
            delete $data->{$url};
        }
        save_plugindata (key_name ($user->id), $data);
    }

    $app->redirect ($app->param('return'));
}

### Hook - pre_run
sub _hdlr_pre_run {
    my ($cb, $app) = @_;

    my $user = $app->user or return;
    my $data = load_plugindata (key_name ($user->id)) || {};
    my $order = 1;

    if (5.0 <= $MT::VERSION) {
        foreach (keys %{$data}) {
            next if !$data->{$_};
            $app->registry ('applications', 'cms', 'menus',
                "bookmark:$order", {
                    label => $data->{$_},
                    order => $order++,
                    link => $_,
                });
        }
    }
    else {
        my $menus = $app->registry ('applications', 'cms', 'menus');
        foreach (keys %{$data}) {
            next if !$data->{$_};
            $menus->{"bookmark:$order"} = {
                    label => $data->{$_},
                    order => $order++,
                    link => $_,
            };
        }
        $app->registry ('applications', 'cms', 'menus', $menus);
    }
}

### Hook - MT::Author::pre_remove
sub _hdlr_pre_remove_author {
    my ($cb, $obj) = @_;

    remove_plugindata (key_name ($obj->id));
}

########################################################################
sub key_name { 'author_id::'. $_[0]; }

sub save_plugindata {
    my ($key, $data_ref) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->id, key=> $key });
    if (!$pd) {
        $pd = MT::PluginData->new;
        $pd->plugin( &instance->id );
        $pd->key( $key );
    }
    $pd->data( $data_ref );
    $pd->save;
}

sub load_plugindata {
    my ($key) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->id, key=> $key })
        or return undef;
    $pd->data;
}

sub remove_plugindata {
    my ($key) = @_;
    my $pd = MT::PluginData->load({ plugin => &instance->id, key=> $key })
        or return undef;# there is no entry all along
    $pd->remove;
}

1;